% [copt,xopt,Zopt,info] = min_sp(F0,Fi,blk,c,opt,x0,Z0)
%
% Solves the primal semidefinite program:
%      Minimize  c'*x      subject to  x1*Fi{1} + .... + xn*Fi{n} < F0  
% and its dual:
%      Maximize  -Tr[F0*Z] subject to  Z=Z' >0 
%                                      -Tr[Fi{k}*Z]=c(k), k=1,...,n
% x is the vector of decision variables and Z is the symmetric matrix
% of dual variables. The function solves the LMI using SP.
%
% Input:
%  F0        m by m symmetric matrix
%  FI        n by 1 cell array of m by m symmetric matrices
%  BLK       A vector containing the dimensions of the 
%               blocks in F0 and Fi.  If F0 and Fi have no block
%               structure, blk = m.  
%  C         m by 1 cost vector 
%  OPT       optional: The key options which can be set are:
%               (1) opt.prt:  NOT USED
%               (2) opt.abstol: Desired absolute accuracy
%               (3) opt.reltol: Desired relative accuracy 
%               (4) opt.maxiter:  Maximum number of iterations 
%               Note 1: This is only a subset of the options for
%               SP. The SP manual lists other
%               options which can be set as follows:  If
%               the SP manual refers to the option 'foo',
%               set this option with 'opt.foo'.
%  X0        optional: Initial primal variable
%  Z0        optional: Initial dual variable
%
% Output:
%  COPT      2x1 vector, COPT(1) is the final primal cost, c'*x,
%            and COPT(2) is the final dual cost, -Tr[F0*Z]
%  XOPT      final value of the vector x of primal variables.
%  ZOPT      final value of dual variables
%  INFO      Additional information from SP:
%            (1) info.info: string from SP with stopping info 
%            (2) info.solvertime: Not an SP output.  This is the
%                cputime actually spent in the SP optimization.
%                This is different from value in info.info because 
%                it includes all time SP takes to set up the 
%                optimization once its function is called.
%            See SP manual for more details. 
%
% See also: 
%
% Coding:   6/30/2001   Pete Seiler

function [copt,xopt,Zopt,info] = min_sp(F0,Fi,blk,c,opt,x0,Z0)

% Is function called correctly?
if nargin <4 | nargout > 4
  error(['usage: [copt,xopt,Zopt,info] = ' ...
    'min_sp(F0,Fi,blk,c,{opt},{x0},{Z0})']);
end
if nargin < 7
  Z0=[];
  if nargin <6
    x0=[];
    if nargin <5
      opt = [];
    end
  end
end

% Is SP installed and in Matlab's search path?
if exist('sp','file')~=3 & exist('sp','file')~=2
  error( sprintf(['SP is not in Matlab''s search path. \n' ... 
  'To download SP, go to: \n' ...
  '\t http://www.ee.ucla.edu/~vandenbe/sp.html']));
end

% Convert from canonical form to SP form.
% Note LMI constraint in SP is : 
%       F0 + x1*F1 + ... + xn*Fn > 0
F = [];base =0;
for i1 = 1:length(blk)
  idx = base+[1:blk(i1)];
  temp = F0(idx,idx);
  F = [F; temp(:)];
  base = base+blk(i1);
end
for i1 = 1:size(Fi,1);
  fi=[];base=0;
  for i2 = 1:length(blk)
    idx = base+[1:blk(i2)];
    temp = Fi{i1}(idx,idx);
    fi = [fi; temp(:)];
    base = base+blk(i2);
  end
  F = [F -fi(:)];
end

% Set SP options  
fldnames = fieldnames(opt);
for i1 =1:length(fldnames)
  eval([fldnames{i1} '= opt.' fldnames{i1} ';']);
end
if ~isfield(opt,'maxiters');   maxiters = opt.maxiter;    end
if ~exist('nu');  nu = 30; end;
if ~exist('tv');    tv=0; end;

% Solve with SP
cputime0=cputime;
if ~isempty(x0) & ~isempty(Z0)
  fprintf('\t\t -------- Running sp.m -------- \n')
  [xopt,Zopt,ul,spinfo,time]= ...
      sp(F,blk,c,x0,Z0,nu,abstol,reltol,tv,maxiters);
elseif ~isempty(x0)  
  % Given primal feasible point, use bigM.m 
  M = 1e5*abs(trace(F0));
  fprintf('\t\t -------- Running bigM.m -------- \n')
  [xopt,Zopt,zopt,ul,iters]= ...
     bigM(F,blk,c,x0,1.1*M,nu,abstol,reltol,tv,maxiters);
  spinfo=[];
else  
  % Use phase1.m to find a primal feasible point
  M = 1e5*abs(trace(F0));
  fprintf('\n\t\t -------- Running phase1.m -------- \n')
  [x0,Z,z,ul,iters]=phase1(F,blk,M,nu,abstol,maxiters);
  if ul(1)>0
    error('Primal problem appears to be infeasable');
  end
  % Given primal feasible point, use bigM.m
  fprintf('\n\t\t -------- Running bigM.m -------- \n')
  temp = trace(F0);
  for i1 = 1:size(Fi,1)
    temp = temp+x0(i1)*trace(Fi{i1});
  end
  M = 1e5*abs(temp);
  [xopt,Zopt,zopt,ul,iters]= ...
     bigM(F,blk,c,x0,M,nu,abstol,reltol,tv,maxiters);
  spinfo=[];
end
info.solvertime = cputime-cputime0;

% Set some output variables
info.info = spinfo;
temp = []; base = 0;
for i1 = 1:length(blk)
  idx = base+[1:blk(i1)^2];
  temp = blkdiag(temp, reshape(Zopt(idx),blk(i1),blk(i1)));
  base = base+blk(i1)^2;
end
Zopt = temp;
if exist('zopt')
  I = eye(size(F0,1));
  copt = [c'*xopt -trace(F0*(Zopt-zopt*I))-M*zopt];
else
  copt = [c'*xopt -trace(F0*Zopt)];
end







